/*
    Парус 8 - Панели мониторинга - Редактор панелей
    Компоненты: Форма (редактор настроек)
*/

//TODO: Контроль уникальности имени элемента формы

//---------------------
//Подключение библиотек
//---------------------

import React, { useEffect, useState, useContext } from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import {
    TextField,
    Button,
    Icon,
    Select,
    MenuItem,
    FormControl,
    InputLabel,
    FormControlLabel,
    Switch,
    Chip,
    Stack,
    InputAdornment,
    IconButton
} from "@mui/material"; //Интерфейсные элементы
import { ApplicationСtx } from "../../../../context/application"; //Контекст приложения
import { STYLES as COMMON_STYLES, EditorBox, EditorSubHeader, ConfigDialog } from "../editors_common"; //Общие компоненты редакторов
import { ITEM_SHAPE, ITEM_INITIAL, ITEMS_INITIAL, ORIENTATION } from "./common"; //Общие ресурсы и константы формы

//---------
//Константы
//---------

//Стили
const STYLES = {
    CHIP_ITEM: { ...COMMON_STYLES.CHIP(true, false) }
};

//------------------------------------
//Вспомогательные функции и компоненты
//------------------------------------

//Редактор элемента
const ItemEditor = ({ item = null, onOk = null, onCancel = null } = {}) => {
    //Собственное состояние - параметры элемента формы
    const [state, setState] = useState({ ...ITEM_INITIAL, ...item });

    //Подключение к контексту приложения
    const { pOnlineShowDictionary } = useContext(ApplicationСtx);

    //При закрытии редактора с сохранением
    const handleOk = () => onOk && onOk({ ...state });

    //При закрытии редактора с отменой
    const handleCancel = () => onCancel && onCancel();

    //При изменении параметра элемента
    const handleChange = e => setState(pv => ({ ...pv, [e.target.id]: e.target.value }));

    //При нажатии на очистку раздела
    const handleClearUnitClick = () =>
        setState(pv => ({
            ...pv,
            unitCode: "",
            unitName: "",
            showMethod: "",
            showMethodName: "",
            parameter: "",
            inputParameter: "",
            outputParameter: ""
        }));

    //При нажатии на выбор раздела
    const handleSelectUnitClick = () => {
        pOnlineShowDictionary({
            unitCode: "Units",
            showMethod: "methods",
            inputParameters: [
                { name: "pos_unit_name", value: state.unitName },
                { name: "pos_method_name", value: state.showMethodName }
            ],
            callBack: res =>
                res.success &&
                setState(pv => ({
                    ...pv,
                    unitCode: res.outParameters.unit_code,
                    unitName: res.outParameters.unit_name,
                    showMethod: res.outParameters.method_code,
                    showMethodName: res.outParameters.method_name,
                    parameter: "",
                    inputParameter: "",
                    outputParameter: ""
                }))
        });
    };

    //При нажатии на выбор параметра метода вызова
    const handleSelectUnitParameterClick = () => {
        state.unitCode &&
            state.showMethod &&
            pOnlineShowDictionary({
                unitCode: "UnitParams",
                showMethod: "main",
                inputParameters: [
                    { name: "in_UNITCODE", value: state.unitCode },
                    { name: "in_PARENT_METHOD_CODE", value: state.showMethod },
                    { name: "in_PARAMNAME", value: state.parameter }
                ],
                callBack: res =>
                    res.success &&
                    setState(pv => ({
                        ...pv,
                        parameter: res.outParameters.out_PARAMNAME,
                        inputParameter: res.outParameters.out_IN_CODE,
                        outputParameter: res.outParameters.out_OUT_CODE
                    }))
            });
    };

    //Формирование представления
    return (
        <ConfigDialog title={`${item ? "Изменение" : "Добавление"} элемента`} onOk={handleOk} onCancel={handleCancel}>
            <Stack direction={"column"} spacing={1}>
                <TextField type={"text"} variant={"standard"} value={state.name} label={"Имя"} id={"name"} onChange={handleChange} />
                <TextField type={"text"} variant={"standard"} value={state.caption} label={"Приглашение"} id={"caption"} onChange={handleChange} />
                <TextField
                    type={"text"}
                    variant={"standard"}
                    value={state.unitName}
                    label={"Раздел"}
                    InputLabelProps={{ shrink: state.unitName ? true : false }}
                    InputProps={{
                        readOnly: true,
                        endAdornment: (
                            <InputAdornment position="end">
                                <IconButton onClick={handleClearUnitClick}>
                                    <Icon>clear</Icon>
                                </IconButton>
                                <IconButton onClick={handleSelectUnitClick}>
                                    <Icon>list</Icon>
                                </IconButton>
                            </InputAdornment>
                        )
                    }}
                />
                <TextField
                    type={"text"}
                    variant={"standard"}
                    value={state.showMethodName}
                    label={"Метод вызова"}
                    InputLabelProps={{ shrink: state.showMethodName ? true : false }}
                    InputProps={{ readOnly: true }}
                />
                <TextField
                    type={"text"}
                    variant={"standard"}
                    value={state.parameter}
                    label={"Параметр"}
                    InputLabelProps={{ shrink: state.parameter ? true : false }}
                    InputProps={{
                        readOnly: true,
                        endAdornment: (
                            <InputAdornment position="end">
                                <IconButton onClick={handleSelectUnitParameterClick}>
                                    <Icon>list</Icon>
                                </IconButton>
                            </InputAdornment>
                        )
                    }}
                />
            </Stack>
        </ConfigDialog>
    );
};

//Контроль свойств - редактор элемента
ItemEditor.propTypes = {
    item: ITEM_SHAPE,
    onOk: PropTypes.func,
    onCancel: PropTypes.func
};

//-----------
//Тело модуля
//-----------

//Форма (редактор настроек)
const FormEditor = ({ id, title = "", orientation = ORIENTATION.V, autoApply = false, items = ITEMS_INITIAL, onSettingsChange = null } = {}) => {
    //Собственное состояние - текущие настройки
    const [settings, setSettings] = useState(null);

    //Собственное состояние - предоставляемые в панель значения
    const [providedValues, setProvidedValues] = useState([]);

    //Собственное состояние - редактор элементов формы
    const [itemEditor, setItemEditor] = useState({ display: false, index: null });

    //При изменении значения настройки
    const handleChange = e => setSettings({ ...settings, [e.target.name]: e.target.type === "checkbox" ? e.target.checked : e.target.value });

    //При добавлении нового элемента
    const handleItemAdd = () => setItemEditor({ display: true, index: null });

    //При нажатии на элемент
    const handleItemClick = i => setItemEditor({ display: true, index: i });

    //При удалении элемента
    const handleItemDelete = i => {
        const items = [...settings.items];
        items.splice(i, 1);
        setSettings(pv => ({ ...pv, items }));
    };

    //При сохранении изменений элемента
    const handleItemSave = item => {
        const items = [...settings.items];
        itemEditor.index == null ? items.push({ ...item }) : (items[itemEditor.index] = { ...item });
        setSettings(pv => ({ ...pv, items }));
        setItemEditor({ display: false, index: null });
    };

    //При отмене сохранения изменений элемента
    const handleItemCancel = () => setItemEditor({ display: false, index: null });

    //При сохранении настроек
    const handleSave = (closeEditor = false) => onSettingsChange && onSettingsChange({ id, settings, providedValues, closeEditor });

    //При изменении компонента
    useEffect(() => {
        settings?.id != id && setSettings({ id, title, orientation, autoApply, items });
    }, [settings, id, title, orientation, autoApply, items]);

    //При изменении состава элементов формы
    useEffect(() => {
        Array.isArray(settings?.items) && setProvidedValues(settings.items.map(item => item.name));
    }, [settings?.items]);

    //Формирование представления
    return (
        settings && (
            <EditorBox title={"Параметры формы"} onSave={handleSave}>
                {itemEditor.display && (
                    <ItemEditor
                        item={itemEditor.index !== null ? { ...settings.items[itemEditor.index] } : null}
                        onCancel={handleItemCancel}
                        onOk={handleItemSave}
                    />
                )}
                <EditorSubHeader title={"Общие"} />
                <TextField type={"text"} variant={"standard"} value={settings.title} label={"Заголовок"} name={"title"} onChange={handleChange} />
                <FormControl variant={"standard"}>
                    <InputLabel id={"orientation-label"}>Ориентация</InputLabel>
                    <Select
                        name={"orientation"}
                        value={settings.orientation}
                        labelId={"orientation-label"}
                        label={"Ориентация"}
                        onChange={handleChange}
                    >
                        <MenuItem value={ORIENTATION.V}>Вертикально</MenuItem>
                        <MenuItem value={ORIENTATION.H}>Горизонтально</MenuItem>
                    </Select>
                </FormControl>
                <FormControlLabel
                    control={<Switch name={"autoApply"} checked={settings.autoApply} onChange={handleChange} />}
                    label={"Автоподтверждение"}
                />
                <EditorSubHeader title={"Элементы"} />
                {Array.isArray(settings?.items) &&
                    settings.items.length > 0 &&
                    settings.items.map((item, i) => (
                        <Chip
                            key={i}
                            label={item.caption}
                            variant={"outlined"}
                            onClick={() => handleItemClick(i)}
                            onDelete={() => handleItemDelete(i)}
                            sx={STYLES.CHIP_ITEM}
                        />
                    ))}
                <Button startIcon={<Icon>add</Icon>} onClick={handleItemAdd}>
                    Добавить элемент
                </Button>
            </EditorBox>
        )
    );
};

//Контроль свойств компонента - Форма (редактор настроек)
FormEditor.propTypes = {
    id: PropTypes.string.isRequired,
    title: PropTypes.string,
    orientation: PropTypes.oneOf(Object.values(ORIENTATION)),
    autoApply: PropTypes.bool,
    items: PropTypes.arrayOf(ITEM_SHAPE),
    onSettingsChange: PropTypes.func
};

//----------------
//Интерфейс модуля
//----------------

export default FormEditor;
